/*
  Der Benutzer bentigt noch das Recht, Oracle Jobs zu definieren. 
  
  Wenn wir z.B. die Installation des Beispiels mit dem Benutzer APEX_XE_PRAXIS 
  durchfhren wollen, so mssen wir als DBA folgendes Recht vergeben: 
  
  grant create job to APEX_XE_PRAXIS;

*/

-- 1) Sequence anlegen
CREATE SEQUENCE rss_feeds_seq
  START WITH 1
  MAXVALUE 999999999999999999999999999
  MINVALUE 1
  NOCYCLE
  NOCACHE
  NOORDER
/

-- 2) Tabelle fr die RSS Feeds, die wir abfragen wollen
CREATE TABLE rss_feeds
(
  rss_id    NUMBER                              NOT NULL,
  rss_name  VARCHAR2(100 BYTE),
  rss_url   VARCHAR2(2000 BYTE)
)
/

-- 3) Primrschlssel
ALTER TABLE rss_feeds ADD (
  CONSTRAINT rss_feeds_pk
 PRIMARY KEY
 (rss_id))
/

-- 4) Tabelle fr die tatschlichen Nachrichten
CREATE TABLE rss_feed_contents
(
  rfc_rss_id       NUMBER,
  rfc_title        VARCHAR2(200 BYTE),
  rfc_url          VARCHAR2(2000 BYTE),
  rfc_display_seq  NUMBER
)
/

-- 5) Trigger fr das automatische Befllen des Primrschlssels 
CREATE OR REPLACE TRIGGER rss_id_bi_trg
   BEFORE INSERT
   ON rss_feeds
   REFERENCING NEW AS NEW OLD AS OLD
   FOR EACH ROW
DECLARE
BEGIN
   SELECT rss_feeds_seq.NEXTVAL
     INTO :NEW.rss_id
     FROM DUAL;
END;
/
SHOW ERRORS;

-- 6) Fremdschlssel zwischen den beiden Tabellen einrichten
ALTER TABLE rss_feed_contents ADD (
  CONSTRAINT rfc_rss_id_fk
 FOREIGN KEY (rfc_rss_id)
 REFERENCES rss_feeds (rss_id))
/

-- Daten fr die ersten RSS Feeds einrichten
INSERT INTO rss_feeds(rss_name, rss_url)
     VALUES ('OTN News',
             'http://www.oracle.com/technology/syndication/rss_otn_news.xml'
            );
INSERT INTO rss_feeds(rss_name, rss_url)
     VALUES ('HTML DB Studio - Most Recent',
             'http://htmldb.oracle.com/pls/otn/studio.recent.rss'
            );
INSERT INTO rss_feeds(rss_name, rss_url)
     VALUES ('HTML DB Studio - Most Popular',
             'http://htmldb.oracle.com/pls/otn/studio.popular.rss'
            );
COMMIT ;

-- Package fr das Befllen der Feed-Tabellen
SET define off;

CREATE OR REPLACE PACKAGE rss_feed
AS
   PROCEDURE refresh_feeds;
END;
/

CREATE OR REPLACE PACKAGE BODY rss_feed
AS
   PROCEDURE refresh_feed (p_rss_id IN NUMBER)
   IS
      l_rss_feed   rss_feeds%ROWTYPE;
   BEGIN
      SELECT *
        INTO l_rss_feed
        FROM rss_feeds
       WHERE rss_id = p_rss_id;

      DELETE FROM rss_feed_contents
            WHERE rfc_rss_id = p_rss_id;

      INSERT INTO rss_feed_contents
                  (rfc_rss_id, rfc_title, rfc_url, rfc_display_seq)
         SELECT p_rss_id, EXTRACTVALUE (VALUE (i), '/item/title') title,
                EXTRACTVALUE (VALUE (i), '/item/link') LINK, NULL
           FROM TABLE
                   (XMLSEQUENCE
                     (EXTRACT (HTTPURITYPE (l_rss_feed.rss_url).getxml(),
                                    '/rss/channel/item'
                                   )
                          )
                   ) i;
   END;

   PROCEDURE refresh_feeds
   IS
   BEGIN
      FOR cur IN (SELECT * FROM rss_feeds)
      LOOP
         refresh_feed (cur.rss_id);
      END LOOP;
   END;
END;
/
SHOW errors;

-- Datenbank-Job einrichten, um alle 15 Minuten die RSS Feeds zu aktualisieren
BEGIN
   DBMS_SCHEDULER.create_job
                (job_name                 => 'RSS_UPDATE_JOB',
                 job_type                 => 'PLSQL_BLOCK',
                 job_action               => 'begin rss_feed.refresh_feeds; 
                                                commit; end;',
                 number_of_arguments      => 0,
                 start_date               => SYSTIMESTAMP,
                 repeat_interval          => 'FREQ=MINUTELY; INTERVAL=15',
                 end_date                 => NULL,
                 job_class                => 'DEFAULT_JOB_CLASS',
                 enabled                  => TRUE,
                 auto_drop                => TRUE,
                 comments                 => NULL
                 );
END;
/
COMMIT ;

